import { Table } from './database.ts';
export type CacheOptions = {
    /** Delete keys automatically after x amount of seconds */
    ttl?: number;
    /** Storage to persist cache */
    storage?: Storage | Table<any, any>;
    /** Key cache will be stored under */
    storageKey?: string;
    /** Keep or delete cached items once expired, defaults to delete */
    expiryPolicy?: 'delete' | 'keep';
};
export type CachedValue<T> = T & {
    _expired?: boolean;
};
/**
 * Map of data which tracks whether it is a complete collection & offers optional expiry of cached values
 */
export declare class Cache<K extends string | number | symbol, T> {
    readonly key?: keyof T | undefined;
    readonly options: CacheOptions;
    private store;
    /** Support index lookups */
    [key: string | number | symbol]: CachedValue<T> | any;
    /** Whether cache is complete */
    complete: boolean;
    /** Await initial loading */
    loading: Promise<void>;
    /**
     * Create new cache
     * @param {keyof T} key Default property to use as primary key
     * @param options
     */
    constructor(key?: keyof T | undefined, options?: CacheOptions);
    private getKey;
    private save;
    /**
     * Get all cached items
     * @return {T[]} Array of items
     */
    all(expired?: boolean): CachedValue<T>[];
    /**
     * Add a new item to the cache. Like set, but finds key automatically
     * @param {T} value Item to add to cache
     * @param {number | undefined} ttl Override default expiry
     * @return {this}
     */
    add(value: T, ttl?: any): this;
    /**
     * Add several rows to the cache
     * @param {T[]} rows Several items that will be cached using the default key
     * @param complete Mark cache as complete & reliable, defaults to true
     * @return {this}
     */
    addAll(rows: T[], complete?: boolean): this;
    /**
     * Remove all keys from cache
     */
    clear(): this;
    /**
     * Delete an item from the cache
     * @param {K} key Item's primary key
     */
    delete(key: K): this;
    /**
     * Return cache as an array of key-value pairs
     * @return {[K, T][]} Key-value pairs array
     */
    entries(expired?: boolean): [K, CachedValue<T>][];
    /**
     * Manually expire a cached item
     * @param {K} key Key to expire
     */
    expire(key: K): this;
    /**
     * Find the first cached item to match a filter
     * @param {Partial<T>} filter Partial item to match
     * @param {Boolean} expired Include expired items, defaults to false
     * @returns {T | undefined} Cached item or undefined if nothing matched
     */
    find(filter: Partial<T>, expired?: boolean): T | undefined;
    /**
     * Get item from the cache
     * @param {K} key Key to lookup
     * @param expired Include expired items
     * @return {T} Cached item
     */
    get(key: K, expired?: boolean): CachedValue<T> | null;
    /**
     * Get a list of cached keys
     * @return {K[]} Array of keys
     */
    keys(expired?: boolean): K[];
    /**
     * Get map of cached items
     * @return {Record<K, T>}
     */
    map(expired?: boolean): Record<K, CachedValue<T>>;
    /**
     * Add an item to the cache manually specifying the key
     * @param {K} key Key item will be cached under
     * @param {T} value Item to cache
     * @param {number | undefined} ttl Override default expiry in seconds
     * @return {this}
     */
    set(key: K, value: T, ttl?: number | undefined): this;
    /**
     * Get all cached items
     * @return {T[]} Array of items
     */
    values: CachedValue<T>[];
}
